@@ -40,10 +40,9 @@
 #define FLOOR(v) ((v) >= 0.0 ? (int) (v) : (int) floor(v))
 
 #define INK8(ink) (*(UINT8*)ink)
-#define INK32(ink) (*(INT32*)ink)
 
 /*
- * Rounds around zero (up=away from zero, down=torwards zero)
+ * Rounds around zero (up=away from zero, down=towards zero)
  * This guarantees that ROUND_UP|DOWN(f) == -ROUND_UP|DOWN(-f)
  */
 #define ROUND_UP(f)    ((int) ((f) >= 0.0 ? floor((f) + 0.5F) : -floor(fabs(f) + 0.5F)))
@@ -67,15 +66,22 @@ typedef void (*hline_handler)(Imaging, int, int, int, int);
 static inline void
 point8(Imaging im, int x, int y, int ink)
 {
-    if (x >= 0 && x < im->xsize && y >= 0 && y < im->ysize)
-        im->image8[y][x] = (UINT8) ink;
+    if (x >= 0 && x < im->xsize && y >= 0 && y < im->ysize) {
+        if (strncmp(im->mode, "I;16", 4) == 0) {
+            im->image8[y][x*2] = (UINT8) ink;
+            im->image8[y][x*2+1] = (UINT8) ink;
+        } else {
+            im->image8[y][x] = (UINT8) ink;
+        }
+    }
 }
 
 static inline void
 point32(Imaging im, int x, int y, int ink)
 {
-    if (x >= 0 && x < im->xsize && y >= 0 && y < im->ysize)
+    if (x >= 0 && x < im->xsize && y >= 0 && y < im->ysize) {
         im->image32[y][x] = ink;
+    }
 }
 
 static inline void
@@ -95,21 +101,27 @@ point32rgba(Imaging im, int x, int y, int ink)
 static inline void
 hline8(Imaging im, int x0, int y0, int x1, int ink)
 {
-    int tmp;
+    int tmp, pixelwidth;
 
     if (y0 >= 0 && y0 < im->ysize) {
-        if (x0 > x1)
+        if (x0 > x1) {
             tmp = x0, x0 = x1, x1 = tmp;
-        if (x0 < 0)
+        }
+        if (x0 < 0) {
             x0 = 0;
-        else if (x0 >= im->xsize)
+        } else if (x0 >= im->xsize) {
             return;
-        if (x1 < 0)
+        }
+        if (x1 < 0) {
             return;
-        else if (x1 >= im->xsize)
+        } else if (x1 >= im->xsize) {
             x1 = im->xsize-1;
-        if (x0 <= x1)
-            memset(im->image8[y0] + x0, (UINT8) ink, x1 - x0 + 1);
+        }
+        if (x0 <= x1) {
+            pixelwidth = strncmp(im->mode, "I;16", 4) == 0 ? 2 : 1;
+            memset(im->image8[y0] + x0 * pixelwidth, (UINT8) ink,
+                   (x1 - x0 + 1) * pixelwidth);
+        }
     }
 }
 
@@ -120,19 +132,23 @@ hline32(Imaging im, int x0, int y0, int x1, int ink)
     INT32* p;
 
     if (y0 >= 0 && y0 < im->ysize) {
-        if (x0 > x1)
+        if (x0 > x1) {
             tmp = x0, x0 = x1, x1 = tmp;
-        if (x0 < 0)
+        }
+        if (x0 < 0) {
             x0 = 0;
-        else if (x0 >= im->xsize)
+        } else if (x0 >= im->xsize) {
             return;
-        if (x1 < 0)
+        }
+        if (x1 < 0) {
             return;
-        else if (x1 >= im->xsize)
+        } else if (x1 >= im->xsize) {
             x1 = im->xsize-1;
+        }
         p = im->image32[y0];
-        while (x0 <= x1)
+        while (x0 <= x1) {
             p[x0++] = ink;
+        }
     }
 }
 
@@ -143,16 +159,19 @@ hline32rgba(Imaging im, int x0, int y0, int x1, int ink)
     unsigned int tmp1;
 
     if (y0 >= 0 && y0 < im->ysize) {
-        if (x0 > x1)
+        if (x0 > x1) {
             tmp = x0, x0 = x1, x1 = tmp;
-        if (x0 < 0)
+        }
+        if (x0 < 0) {
             x0 = 0;
-        else if (x0 >= im->xsize)
+        } else if (x0 >= im->xsize) {
             return;
-        if (x1 < 0)
+        }
+        if (x1 < 0) {
             return;
-        else if (x1 >= im->xsize)
+        } else if (x1 >= im->xsize) {
             x1 = im->xsize-1;
+        }
         if (x0 <= x1) {
             UINT8* out = (UINT8*) im->image[y0]+x0*4;
             UINT8* in = (UINT8*) &ink;
@@ -175,19 +194,21 @@ line8(Imaging im, int x0, int y0, int x1, int y1, int ink)
 
     /* normalize coordinates */
     dx = x1-x0;
-    if (dx < 0)
+    if (dx < 0) {
         dx = -dx, xs = -1;
-    else
+    } else {
         xs = 1;
+    }
     dy = y1-y0;
-    if (dy < 0)
+    if (dy < 0) {
         dy = -dy, ys = -1;
-    else
+    } else {
         ys = 1;
+    }
 
     n = (dx > dy) ? dx : dy;
 
-    if (dx == 0)
+    if (dx == 0) {
 
         /* vertical */
         for (i = 0; i < dy; i++) {
@@ -195,7 +216,7 @@ line8(Imaging im, int x0, int y0, int x1, int y1, int ink)
             y0 += ys;
         }
 
-    else if (dy == 0)
+    } else if (dy == 0) {
 
         /* horizontal */
         for (i = 0; i < dx; i++) {
@@ -203,7 +224,7 @@ line8(Imaging im, int x0, int y0, int x1, int y1, int ink)
             x0 += xs;
         }
 
-    else if (dx > dy) {
+    } else if (dx > dy) {
 
         /* bresenham, horizontal slope */
         n = dx;
@@ -251,19 +272,21 @@ line32(Imaging im, int x0, int y0, int x1, int y1, int ink)
 
     /* normalize coordinates */
     dx = x1-x0;
-    if (dx < 0)
+    if (dx < 0) {
         dx = -dx, xs = -1;
-    else
+    } else {
         xs = 1;
+    }
     dy = y1-y0;
-    if (dy < 0)
+    if (dy < 0) {
         dy = -dy, ys = -1;
-    else
+    } else {
         ys = 1;
+    }
 
     n = (dx > dy) ? dx : dy;
 
-    if (dx == 0)
+    if (dx == 0) {
 
         /* vertical */
         for (i = 0; i < dy; i++) {
@@ -271,7 +294,7 @@ line32(Imaging im, int x0, int y0, int x1, int y1, int ink)
             y0 += ys;
         }
 
-    else if (dy == 0)
+    } else if (dy == 0) {
 
         /* horizontal */
         for (i = 0; i < dx; i++) {
@@ -279,7 +302,7 @@ line32(Imaging im, int x0, int y0, int x1, int y1, int ink)
             x0 += xs;
         }
 
-    else if (dx > dy) {
+    } else if (dx > dy) {
 
         /* bresenham, horizontal slope */
         n = dx;
@@ -327,19 +350,21 @@ line32rgba(Imaging im, int x0, int y0, int x1, int y1, int ink)
 
     /* normalize coordinates */
     dx = x1-x0;
-    if (dx < 0)
+    if (dx < 0) {
         dx = -dx, xs = -1;
-    else
+    } else {
         xs = 1;
+    }
     dy = y1-y0;
-    if (dy < 0)
+    if (dy < 0) {
         dy = -dy, ys = -1;
-    else
+    } else {
         ys = 1;
+    }
 
     n = (dx > dy) ? dx : dy;
 
-    if (dx == 0)
+    if (dx == 0) {
 
         /* vertical */
         for (i = 0; i < dy; i++) {
@@ -347,7 +372,7 @@ line32rgba(Imaging im, int x0, int y0, int x1, int y1, int ink)
             y0 += ys;
         }
 
-    else if (dy == 0)
+    } else if (dy == 0) {
 
         /* horizontal */
         for (i = 0; i < dx; i++) {
@@ -355,7 +380,7 @@ line32rgba(Imaging im, int x0, int y0, int x1, int y1, int ink)
             x0 += xs;
         }
 
-    else if (dx > dy) {
+    } else if (dx > dy) {
 
         /* bresenham, horizontal slope */
         n = dx;
@@ -398,15 +423,45 @@ static int
 x_cmp(const void *x0, const void *x1)
 {
     float diff = *((float*)x0) - *((float*)x1);
-    if (diff < 0)
+    if (diff < 0) {
         return -1;
-    else if (diff > 0)
+    } else if (diff > 0) {
         return 1;
-    else
+    } else {
         return 0;
+    }
 }
 
 
+static void
+draw_horizontal_lines(Imaging im, int n, Edge *e, int ink, int *x_pos, int y, hline_handler hline)
+{
+    int i;
+    for (i = 0; i < n; i++) {
+        if (e[i].ymin == y && e[i].ymin == e[i].ymax) {
+            int xmax;
+            int xmin = e[i].xmin;
+            if (*x_pos < xmin) {
+                // Line would be after the current position
+                continue;
+            }
+
+            xmax = e[i].xmax;
+            if (*x_pos > xmin) {
+                // Line would be partway through x_pos, so increase the starting point
+                xmin = *x_pos;
+                if (xmax < xmin) {
+                    // Line would now end before it started
+                    continue;
+                }
+            }
+
+            (*hline)(im, xmin, e[i].ymin, xmax, ink);
+            *x_pos = xmax+1;
+        }
+    }
+}
+
 /*
  * Filled polygon draw function using scan line algorithm.
  */
@@ -434,18 +489,15 @@ polygon_generic(Imaging im, int n, Edge *e, int ink, int eofill,
     }
 
     for (i = 0; i < n; i++) {
-        /* This causes the pixels of horizontal edges to be drawn twice :(
-         * but without it there are inconsistencies in ellipses */
-        if (e[i].ymin == e[i].ymax) {
-            (*hline)(im, e[i].xmin, e[i].ymin, e[i].xmax, ink);
-            continue;
-        }
         if (ymin > e[i].ymin) {
             ymin = e[i].ymin;
         }
         if (ymax < e[i].ymax) {
             ymax = e[i].ymax;
         }
+        if (e[i].ymin == e[i].ymax) {
+            continue;
+        }
         edge_table[edge_count++] = (e + i);
     }
     if (ymin < 0) {
@@ -464,6 +516,7 @@ polygon_generic(Imaging im, int n, Edge *e, int ink, int eofill,
     }
     for (; ymin <= ymax; ymin++) {
         int j = 0;
+        int x_pos = 0;
         for (i = 0; i < edge_count; i++) {
             Edge* current = edge_table[i];
             if (ymin >= current->ymin && ymin <= current->ymax) {
@@ -477,8 +530,30 @@ polygon_generic(Imaging im, int n, Edge *e, int ink, int eofill,
         }
         qsort(xx, j, sizeof(float), x_cmp);
         for (i = 1; i < j; i += 2) {
-            (*hline)(im, ROUND_UP(xx[i - 1]), ymin, ROUND_DOWN(xx[i]), ink);
+            int x_end = ROUND_DOWN(xx[i]);
+            if (x_end < x_pos) {
+                // Line would be before the current position
+                continue;
+            }
+            draw_horizontal_lines(im, n, e, ink, &x_pos, ymin, hline);
+            if (x_end < x_pos) {
+                // Line would be before the current position
+                continue;
+            }
+
+            int x_start = ROUND_UP(xx[i-1]);
+            if (x_pos > x_start) {
+                // Line would be partway through x_pos, so increase the starting point
+                x_start = x_pos;
+                if (x_end < x_start) {
+                    // Line would now end before it started
+                    continue;
+                }
+            }
+            (*hline)(im, x_start, ymin, x_end, ink);
+            x_pos = x_end+1;
         }
+        draw_horizontal_lines(im, n, e, ink, &x_pos, ymin, hline);
     }
 
     free(xx);
@@ -509,25 +584,28 @@ add_edge(Edge *e, int x0, int y0, int x1, int y1)
 {
     /* printf("edge %d %d %d %d\n", x0, y0, x1, y1); */
 
-    if (x0 <= x1)
+    if (x0 <= x1) {
         e->xmin = x0, e->xmax = x1;
-    else
+    } else {
         e->xmin = x1, e->xmax = x0;
+    }
 
-    if (y0 <= y1)
+    if (y0 <= y1) {
         e->ymin = y0, e->ymax = y1;
-    else
+    } else {
         e->ymin = y1, e->ymax = y0;
+    }
 
     if (y0 == y1) {
         e->d = 0;
         e->dx = 0.0;
     } else {
         e->dx = ((float)(x1-x0)) / (y1-y0);
-        if (y0 == e->ymin)
+        if (y0 == e->ymin) {
             e->d = 1;
-        else
+        } else {
             e->d = -1;
+        }
     }
 
     e->x0 = x0;
@@ -555,7 +633,7 @@ DRAW draw32rgba = { point32rgba, hline32rgba, line32rgba, polygon32rgba };
         ink = INK8(ink_);\
     } else {\
         draw = (op) ? &draw32rgba : &draw32;    \
-        ink = INK32(ink_);\
+        memcpy(&ink, ink_, sizeof(ink)); \
     }
 
 int
@@ -605,7 +683,7 @@ ImagingDrawWideLine(Imaging im, int x0, int y0, int x1, int y1,
         return 0;
     }
 
-    big_hypotenuse = sqrt((double) (dx*dx + dy*dy));
+    big_hypotenuse = hypot(dx, dy);
     small_hypotenuse = (width - 1) / 2.0;
     ratio_max = ROUND_UP(small_hypotenuse) / big_hypotenuse;
     ratio_min = ROUND_DOWN(small_hypotenuse) / big_hypotenuse;
@@ -644,23 +722,27 @@ ImagingDrawRectangle(Imaging im, int x0, int y0, int x1, int y1,
 
     DRAWINIT();
 
-    if (y0 > y1)
+    if (y0 > y1) {
         tmp = y0, y0 = y1, y1 = tmp;
+    }
 
     if (fill) {
 
-        if (y0 < 0)
+        if (y0 < 0) {
             y0 = 0;
-        else if (y0 >= im->ysize)
+        } else if (y0 >= im->ysize) {
             return 0;
+        }
 
-        if (y1 < 0)
+        if (y1 < 0) {
             return 0;
-        else if (y1 > im->ysize)
+        } else if (y1 > im->ysize) {
             y1 = im->ysize;
+        }
 
-        for (y = y0; y <= y1; y++)
+        for (y = y0; y <= y1; y++) {
             draw->hline(im, x0, y, x1, ink);
+        }
 
     } else {
         /* outline */
@@ -686,8 +768,9 @@ ImagingDrawPolygon(Imaging im, int count, int* xy, const void* ink_,
     DRAW* draw;
     INT32 ink;
 
-    if (count <= 0)
+    if (count <= 0) {
         return 0;
+    }
 
     DRAWINIT();
 
@@ -700,18 +783,21 @@ ImagingDrawPolygon(Imaging im, int count, int* xy, const void* ink_,
             (void) ImagingError_MemoryError();
             return -1;
         }
-        for (i = n = 0; i < count-1; i++)
+        for (i = n = 0; i < count-1; i++) {
             add_edge(&e[n++], xy[i+i], xy[i+i+1], xy[i+i+2], xy[i+i+3]);
-        if (xy[i+i] != xy[0] || xy[i+i+1] != xy[1])
+        }
+        if (xy[i+i] != xy[0] || xy[i+i+1] != xy[1]) {
             add_edge(&e[n++], xy[i+i], xy[i+i+1], xy[0], xy[1]);
+        }
         draw->polygon(im, n, e, ink, 0);
         free(e);
 
     } else {
 
         /* Outline */
-        for (i = 0; i < count-1; i++)
+        for (i = 0; i < count-1; i++) {
             draw->line(im, xy[i+i], xy[i+i+1], xy[i+i+2], xy[i+i+3], ink);
+        }
         draw->line(im, xy[i+i], xy[i+i+1], xy[0], xy[1], ink);
 
     }
@@ -765,115 +851,156 @@ ellipse(Imaging im, int x0, int y0, int x1, int y1,
         int width, int mode, int op)
 {
     float i;
-    int j;
+    int inner;
     int n;
-    int cx, cy;
+    int maxEdgeCount;
     int w, h;
-    int x = 0, y = 0;
+    int x, y;
+    int cx, cy;
     int lx = 0, ly = 0;
     int sx = 0, sy = 0;
+    int lx_inner = 0, ly_inner = 0;
+    int sx_inner = 0, sy_inner = 0;
     DRAW* draw;
     INT32 ink;
+    Edge* e;
 
     DRAWINIT();
 
-    if (width == 0) {
-        width = 1;
+    while (end < start) {
+        end += 360;
     }
 
-    for (j = 0; j < width; j++) {
+    if (end - start > 360) {
+        // no need to go in loops
+        end = start + 361;
+    }
 
-        w = x1 - x0;
-        h = y1 - y0;
-        if (w < 0 || h < 0)
-            return 0;
+    w = x1 - x0;
+    h = y1 - y0;
+    if (w <= 0 || h <= 0) {
+        return 0;
+    }
 
-        cx = (x0 + x1) / 2;
-        cy = (y0 + y1) / 2;
+    cx = (x0 + x1) / 2;
+    cy = (y0 + y1) / 2;
 
-        while (end < start)
-            end += 360;
-
-        if (end - start > 360) {
-            /* no need to go in loops */
-            end = start + 361;
-        }
-
-        if (mode != ARC && fill) {
-
-            /* Build edge list */
-            /* malloc check UNDONE, FLOAT? */
-            Edge* e = calloc((end - start + 3), sizeof(Edge));
-            if (!e) {
-                ImagingError_MemoryError();
-                return -1;
+    if (!fill && width <= 1) {
+        for (i = start; i < end+1; i++) {
+            if (i > end) {
+                i = end;
             }
-            n = 0;
-
-            for (i = start; i < end+1; i++) {
-                if (i > end) {
-                    i = end;
-                }
-                ellipsePoint(cx, cy, w, h, i, &x, &y);
-                if (i != start)
-                    add_edge(&e[n++], lx, ly, x, y);
-                else
-                    sx = x, sy = y;
-                lx = x, ly = y;
-            }
-
-            if (n > 0) {
-                /* close and draw polygon */
-                if (mode == PIESLICE) {
-                    if (x != cx || y != cy) {
-                        add_edge(&e[n++], x, y, cx, cy);
-                        add_edge(&e[n++], cx, cy, sx, sy);
-                    }
-                } else {
-                    if (x != sx || y != sy)
-                        add_edge(&e[n++], x, y, sx, sy);
-                }
-                draw->polygon(im, n, e, ink, 0);
-            }
-
-            free(e);
-
-        } else {
-
-            for (i = start; i < end+1; i++) {
-                if (i > end) {
-                    i = end;
-                }
-                ellipsePoint(cx, cy, w, h, i, &x, &y);
-                if (i != start)
-                    draw->line(im, lx, ly, x, y, ink);
-                else
-                    sx = x, sy = y;
-                lx = x, ly = y;
-            }
-
+            ellipsePoint(cx, cy, w, h, i, &x, &y);
             if (i != start) {
-                if (mode == PIESLICE) {
-                    if (j == 0 && (x != cx || y != cy)) {
-                        if (width == 1) {
-                            draw->line(im, x, y, cx, cy, ink);
-                            draw->line(im, cx, cy, sx, sy, ink);
-                        } else {
-                            ImagingDrawWideLine(im, x, y, cx, cy, &ink, width, op);
-                            ImagingDrawWideLine(im, cx, cy, sx, sy, &ink, width, op);
-                        }
-                    }
-                } else if (mode == CHORD) {
-                    if (x != sx || y != sy)
-                        draw->line(im, x, y, sx, sy, ink);
+                draw->line(im, lx, ly, x, y, ink);
+            } else {
+                sx = x, sy = y;
+            }
+            lx = x, ly = y;
+        }
+
+        if (i != start) {
+            if (mode == PIESLICE) {
+                if (x != cx || y != cy) {
+                    draw->line(im, x, y, cx, cy, ink);
+                    draw->line(im, cx, cy, sx, sy, ink);
+                }
+            } else if (mode == CHORD) {
+                if (x != sx || y != sy) {
+                    draw->line(im, x, y, sx, sy, ink);
                 }
             }
         }
-        x0++;
-        y0++;
-        x1--;
-        y1--;
+    } else {
+        inner = (mode == ARC || !fill) ? 1 : 0;
+
+        // Build edge list
+        // malloc check UNDONE, FLOAT?
+        maxEdgeCount = ceil(end - start);
+        if (inner) {
+            maxEdgeCount *= 2;
+        }
+        maxEdgeCount += 3;
+        e = calloc(maxEdgeCount, sizeof(Edge));
+        if (!e) {
+            ImagingError_MemoryError();
+            return -1;
+        }
+
+        // Outer circle
+        n = 0;
+        for (i = start; i < end+1; i++) {
+            if (i > end) {
+                i = end;
+            }
+            ellipsePoint(cx, cy, w, h, i, &x, &y);
+            if (i == start) {
+                sx = x, sy = y;
+            } else {
+                add_edge(&e[n++], lx, ly, x, y);
+            }
+            lx = x, ly = y;
+        }
+        if (n == 0) {
+            return 0;
+        }
+
+        if (inner) {
+            // Inner circle
+            x0 += width - 1;
+            y0 += width - 1;
+            x1 -= width - 1;
+            y1 -= width - 1;
+
+            w = x1 - x0;
+            h = y1 - y0;
+            if (w <= 0 || h <= 0) {
+                // ARC with no gap in the middle is a PIESLICE
+                mode = PIESLICE;
+                inner = 0;
+            } else {
+                for (i = start; i < end+1; i++) {
+                    if (i > end) {
+                        i = end;
+                    }
+                    ellipsePoint(cx, cy, w, h, i, &x, &y);
+                    if (i == start) {
+                        sx_inner = x, sy_inner = y;
+                    } else {
+                        add_edge(&e[n++], lx_inner, ly_inner, x, y);
+                    }
+                    lx_inner = x, ly_inner = y;
+                }
+            }
+        }
+
+        if (end - start < 360) {
+            // Close polygon
+            if (mode == PIESLICE) {
+                if (x != cx || y != cy) {
+                    add_edge(&e[n++], sx, sy, cx, cy);
+                    add_edge(&e[n++], cx, cy, lx, ly);
+                    if (inner) {
+                        ImagingDrawWideLine(im, sx, sy, cx, cy, &ink, width, op);
+                        ImagingDrawWideLine(im, cx, cy, lx, ly, &ink, width, op);
+                    }
+                }
+            } else if (mode == CHORD) {
+                add_edge(&e[n++], sx, sy, lx, ly);
+                if (inner) {
+                    add_edge(&e[n++], sx_inner, sy_inner, lx_inner, ly_inner);
+                }
+            } else if (mode == ARC) {
+                add_edge(&e[n++], sx, sy, sx_inner, sy_inner);
+                add_edge(&e[n++], lx, ly, lx_inner, ly_inner);
+            }
+        }
+
+        draw->polygon(im, n, e, ink, 0);
+
+        free(e);
     }
+
     return 0;
 }
 
@@ -934,8 +1061,9 @@ ImagingOutlineNew(void)
     ImagingOutline outline;
 
     outline = calloc(1, sizeof(struct ImagingOutlineInstance));
-    if (!outline)
+    if (!outline) {
         return (ImagingOutline) ImagingError_MemoryError();
+    }
 
     outline->edges = NULL;
     outline->count = outline->size = 0;
@@ -948,11 +1076,13 @@ ImagingOutlineNew(void)
 void
 ImagingOutlineDelete(ImagingOutline outline)
 {
-    if (!outline)
+    if (!outline) {
         return;
+    }
 
-    if (outline->edges)
+    if (outline->edges) {
         free(outline->edges);
+    }
 
     free(outline);
 }
@@ -976,8 +1106,9 @@ allocate(ImagingOutline outline, int extra)
             /* malloc check ok, overflow checked above */
             e = realloc(outline->edges, outline->size * sizeof(Edge));
         }
-        if (!e)
+        if (!e) {
             return NULL;
+        }
         outline->edges = e;
     }
 
@@ -1003,8 +1134,9 @@ ImagingOutlineLine(ImagingOutline outline, float x1, float y1)
     Edge* e;
 
     e = allocate(outline, 1);
-    if (!e)
+    if (!e) {
         return -1; /* out of memory */
+    }
 
     add_edge(e, (int) outline->x, (int) outline->y, (int) x1, (int) y1);
 
@@ -1025,8 +1157,9 @@ ImagingOutlineCurve(ImagingOutline outline, float x1, float y1,
 #define STEPS 32
 
     e = allocate(outline, STEPS);
-    if (!e)
+    if (!e) {
         return -1; /* out of memory */
+    }
 
     xo = outline->x;
     yo = outline->y;
@@ -1061,8 +1194,9 @@ ImagingOutlineCurve(ImagingOutline outline, float x1, float y1,
 int
 ImagingOutlineClose(ImagingOutline outline)
 {
-    if (outline->x == outline->x0 && outline->y == outline->y0)
+    if (outline->x == outline->x0 && outline->y == outline->y0) {
         return 0;
+    }
     return ImagingOutlineLine(outline, outline->x0, outline->y0);
 }
 
@@ -1099,14 +1233,16 @@ ImagingOutlineTransform(ImagingOutline outline, double a[6])
         y0 = eIn->y0;
 
         /* FIXME: ouch! */
-        if (eIn->x0 == eIn->xmin)
+        if (eIn->x0 == eIn->xmin) {
             x1 = eIn->xmax;
-        else
+        } else {
             x1 = eIn->xmin;
-        if (eIn->y0 == eIn->ymin)
+        }
+        if (eIn->y0 == eIn->ymin) {
             y1 = eIn->ymax;
-        else
+        } else {
             y1 = eIn->ymin;
+        }
 
         /* full moon tonight!  if this doesn't work, you may need to
            upgrade your compiler (make sure you have the right service
